/***************************************************/
/* Game Excelleration Kit vX                       */
/* Copyright (C) 2001 by HourGlass Software NonInc */
/***************************************************/
#ifndef __GEKQuaternion__
#define __GEKQuaternion__
/*! 
    \author MoJo
    \file GEKQuaternion.h
    \brief 4D Quaternion Class
*/

//! 4D Quaternion Class
struct GEKQUATERNION
{
public:
	float x;
	float y;
	float z;
	float w;

public:
	inline GEKQUATERNION(void);
	inline GEKQUATERNION(const GEKMATRIX &kMatrix);
	inline GEKQUATERNION(const GEKQUATERNION &kQuat);
	inline GEKQUATERNION(float fX, float fY, float fZ, float fW);
	inline GEKQUATERNION(const GEKVECTOR3 &kAxis, float fAngle);
	inline GEKQUATERNION(const GEKVECTOR3 &kEuler);

	//! Casting Operator
	inline operator float * (void);

	//@{
	//! Assignment Operators
	void operator *= (const GEKQUATERNION &kQuat);
	inline GEKQUATERNION operator = (const GEKQUATERNION &kQuat);
	inline operator -= (const GEKQUATERNION &kQuat);
	inline operator += (const GEKQUATERNION &kQuat);
	inline operator /= (float fScalar);
	inline operator *= (float fScalar);
	//@}

	//@{
	//! Binary Operators
	GEKQUATERNION operator * (const GEKQUATERNION &kQuat);
	inline GEKQUATERNION operator - (const GEKQUATERNION &kQuat);
	inline GEKQUATERNION operator + (const GEKQUATERNION &kQuat);
	inline GEKQUATERNION operator / (float fScalar);
	inline GEKQUATERNION operator * (float fScalar);
	//@}

	inline void SetIdentity(void);								//!< Sets a quaternion to identity
	void SetEuler(const GEKVECTOR3 &kEuler);					//!< Sets the XYZ euler angles of a quaternion
	void SetRotation(const GEKVECTOR3 &kAxis, float fAngle);	//!< Sets the axial angle of a quaternion
	void SetMatrix(const GEKMATRIX &kMatrix);					//!< Converts a rotation matrix to a quaternion
	
	inline float Dot(const GEKQUATERNION &kQuat);				//!< Calculates the dot product of two quaternions
	inline float LengthSqr(void);								//!< Calculates the square length of a quaternion
	inline float Length(void);									//!< Calculates the length of a quaternion

	GEKQUATERNION Normal(void);									//!< Calculates the normal of a quaternion
	void Normalize(void);										//!< Normalizes a quaternion

	GEKQUATERNION Slerp(const GEKQUATERNION &kQuat, float fFactor);	//!< Calculates the spherical interpolation of two quaternions
	GEKQUATERNION Lerp(const GEKQUATERNION &kQuat, float fFactor);	//!< Calculates the linear interpolation of two quaternions
};

#endif