/***************************************************/
/* Game Excelleration Kit vX                       */
/* Copyright (C) 2001 by HourGlass Software NonInc */
/***************************************************/
#ifndef __GEKVECTOR3__
#define __GEKVECTOR3__
/*! 
    \author MoJo
    \file GEKVector3.h
    \brief 3D Vector Class
*/

//! 3D Vector Class
struct GEKVECTOR3
{
public:
	union
	{
		float xyz[3];
		struct { float x, y, z; };
	};

public:
	inline GEKVECTOR3(void);
	inline GEKVECTOR3(const GEKVECTOR2 &kVector);
	inline GEKVECTOR3(const GEKVECTOR3 &kVector);
	inline GEKVECTOR3(const GEKVECTOR4 &kVector);
	inline GEKVECTOR3(float fX, float fY, float fZ);

	inline float &operator [] (int iAxis);
	inline bool operator < (const GEKVECTOR3 &kVector) const;
	inline bool operator > (const GEKVECTOR3 &kVector) const;

	//@{
	//! Casting operator
	inline operator float * (void);
	inline operator const float * (void) const;
	//@}

	//@{
	//! Assignment operators
	inline GEKVECTOR3 operator = (const GEKVECTOR2 &kVector);
	inline GEKVECTOR3 operator = (const GEKVECTOR3 &kVector);
	inline GEKVECTOR3 operator = (const GEKVECTOR4 &kVector);
	inline operator -= (const GEKVECTOR3 &kVector);
	inline operator += (const GEKVECTOR3 &kVector);
	inline operator /= (float fScalar);
	inline operator *= (float fScalar);
	//@}

	//@{
	//! Binary Operators
	inline GEKVECTOR3 operator - (const GEKVECTOR3 &kVector) const;
	inline GEKVECTOR3 operator + (const GEKVECTOR3 &kVector) const;
	inline GEKVECTOR3 operator - (const GEKVECTOR3 &kVector);
	inline GEKVECTOR3 operator + (const GEKVECTOR3 &kVector);
	inline GEKVECTOR3 operator / (float fScalar) const;
	inline GEKVECTOR3 operator * (float fScalar) const;
	inline GEKVECTOR3 operator / (float fScalar);
	inline GEKVECTOR3 operator * (float fScalar);
	//@}

	inline void SetLength(float fLength);						//!< Sets the length of a vector
	inline float GetLengthSqr(void) const;						//!< Calculates the square length of a vector
	inline float GetLengthSqr(void);							//!< Calculates the square length of a vector
	inline float GetLength(void) const;							//!< Calculates the length of a vector
	inline float GetLength(void);								//!< Calculates the length of a vector

	inline float GetAngle(const GEKVECTOR3 &kVector);		//!< Calculates the angle between two vectors
	inline GEKVECTOR3 Cross(const GEKVECTOR3 &kVector) const;	//!< Calculates the cross product of two vectors
	inline GEKVECTOR3 Cross(const GEKVECTOR3 &kVector);			//!< Calculates the cross product of two vectors
	inline float Distance(const GEKVECTOR3 &kVector) const;		//!< Calculates the distance to a vector
	inline float Distance(const GEKVECTOR3 &kVector);			//!< Calculates the distance to a vector
	inline float Dot(const GEKVECTOR3 &kVector) const;			//!< Calculates the dot product of two vectors
	inline float Dot(const GEKVECTOR3 &kVector);				//!< Calculates the dot product of two vectors

	inline GEKVECTOR3 Lerp(const GEKVECTOR3 &kVector, float fFactor) const;		//!< Calculates the linear interpolation of two vectors
	inline GEKVECTOR3 Lerp(const GEKVECTOR3 &kVector, float fFactor);			//!< Calculates the linear interpolation of two vectors
	inline GEKVECTOR3 Normal(void) const;										//!< Calculates the normal of a vector
	inline GEKVECTOR3 Normal(void);												//!< Calculates the normal of a vector
	inline void Normalize(void);												//!< Normalizes a vector
};

inline GEKVECTOR3 operator - (const GEKVECTOR3 &kVector);	//!< Negative Operator
inline GEKVECTOR3 operator - (GEKVECTOR3 &kVector);			//!< Negative Operator

#endif