inline GEKQUATERNION::GEKQUATERNION(void)
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
	w = 1.0f;
}

inline GEKQUATERNION::GEKQUATERNION(const GEKMATRIX &kMatrix)
{
	SetMatrix(kMatrix);
}

inline GEKQUATERNION::GEKQUATERNION(const GEKQUATERNION &kQuat)
{
	x = kQuat.x;
	y = kQuat.y;
	z = kQuat.z;
	w = kQuat.w;
}

inline GEKQUATERNION::GEKQUATERNION(float fX, float fY, float fZ, float fW)
{
	x = fX;
	y = fY;
	z = fZ;
	w = fW;
}

inline GEKQUATERNION::GEKQUATERNION(const GEKVECTOR3 &kAxis, float fAngle)
{
	SetRotation(kAxis, fAngle);
}

inline GEKQUATERNION::GEKQUATERNION(const GEKVECTOR3 &kEuler)
{
	SetEuler(kEuler);
}

inline GEKQUATERNION::operator float * (void)
{
	return &x;
}

inline GEKQUATERNION::operator -= (const GEKQUATERNION &kQuat)
{
	x -= kQuat.x;
	y -= kQuat.y;
	z -= kQuat.z;
	w -= kQuat.w;
}

inline GEKQUATERNION::operator += (const GEKQUATERNION &kQuat)
{
	x += kQuat.x;
	y += kQuat.y;
	z += kQuat.z;
	w += kQuat.w;
}

inline GEKQUATERNION::operator /= (float fScalar)
{
	x /= fScalar;
	y /= fScalar;
	z /= fScalar;
	w /= fScalar;
}

inline GEKQUATERNION::operator *= (float fScalar)
{
	x *= fScalar;
	y *= fScalar;
	z *= fScalar;
	w *= fScalar;
}

inline GEKQUATERNION GEKQUATERNION::operator = (const GEKQUATERNION &kQuat)
{
	x = kQuat.x;
	y = kQuat.y;
	z = kQuat.z;
	w = kQuat.w;
	return *this;
}

inline GEKQUATERNION GEKQUATERNION::operator - (const GEKQUATERNION &kQuat)
{
	return GEKQUATERNION((x - kQuat.x),
					    (y - kQuat.y),
					    (z - kQuat.z),
					    (w - kQuat.w));
}

inline GEKQUATERNION GEKQUATERNION::operator + (const GEKQUATERNION &kQuat)
{
	return GEKQUATERNION((x + kQuat.x),
					    (y + kQuat.y),
					    (z + kQuat.z),
					    (w + kQuat.w));
}

inline GEKQUATERNION GEKQUATERNION::operator / (float fScalar)
{
	return GEKQUATERNION((x / fScalar),
				  		 (y / fScalar),
						 (z / fScalar),
						 (w / fScalar));
}

inline GEKQUATERNION GEKQUATERNION::operator * (float fScalar)
{
	return GEKQUATERNION((x * fScalar),
						 (y * fScalar),
						 (z * fScalar),
						 (w * fScalar));
}

inline void GEKQUATERNION::SetIdentity(void)
{
	x = y = z = 0.0f;
	w = 1.0f;
}

inline float GEKQUATERNION::Dot(const GEKQUATERNION &kQuat)
{
	return ((x * kQuat.x) +
			(y * kQuat.y) +
			(z * kQuat.z) +
			(w * kQuat.w));
}

inline float GEKQUATERNION::LengthSqr(void)
{
	return Dot(*this);
}

inline float GEKQUATERNION::Length(void)
{
	return SQRT(LengthSqr());
}